Package: vetr
Title: Trust, but Verify
Description: Declarative template-based framework for verifying that objects
  meet structural requirements, and auto-composing error messages when they do
  not.
Version: 0.2.22
Authors@R: c(
    person("Brodie", "Gaslam", email="brodie.gaslam@yahoo.com",
    role=c("aut", "cre")),
    person(given="Paxdiablo", role="cph", comment="Hash
    table implementation in src/pfhash.h"),
    person(given="R Core Team",
    email="R-core@r-project.org", role="cph",
    comment="Used/adapted several code snippets from R sources, see src/r-copied.c"),
    person("Michael","Chirico", role="ctb",
    email="michaelchirico4@gmail.com", comment = c(ORCID="0000-0003-0787-087X")
    ))
Depends: R (>= 3.2.0)
License: GPL (>= 2)
URL: https://github.com/brodieG/vetr
BugReports: https://github.com/brodieG/vetr/issues
VignetteBuilder: knitr
Imports: methods, stats, utils
Suggests: knitr, rmarkdown, unitizer
RoxygenNote: 7.3.3
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-04-27 11:33:31 UTC; brodie
Author: Brodie Gaslam [aut, cre],
  Paxdiablo [cph] (Hash table implementation in src/pfhash.h),
  R Core Team [cph] (Used/adapted several code snippets from R sources,
    see src/r-copied.c),
  Michael Chirico [ctb] (ORCID: <https://orcid.org/0000-0003-0787-087X>)
Maintainer: Brodie Gaslam <brodie.gaslam@yahoo.com>
Repository: CRAN
Date/Publication: 2026-04-27 13:50:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-01 17:08:21 UTC; unix
Archs: vetr.so.dSYM
