# Generated by roxygen2: do not edit by hand

S3method(abstract,array)
S3method(abstract,data.frame)
S3method(abstract,default)
S3method(abstract,environment)
S3method(abstract,ggplot)
S3method(abstract,list)
S3method(abstract,lm)
S3method(abstract,matrix)
S3method(abstract,ts)
S3method(nullify,default)
export(CHR)
export(CHR.1)
export(CPX)
export(CPX.1)
export(GT.0)
export(GTE.0)
export(INT)
export(INT.1)
export(INT.1.NEG)
export(INT.1.NEG.STR)
export(INT.1.POS)
export(INT.1.POS.STR)
export(INT.NEG)
export(INT.NEG.STR)
export(INT.POS)
export(INT.POS.STR)
export(LGL)
export(LGL.1)
export(LT.0)
export(LTE.0)
export(NO.INF)
export(NO.NA)
export(NUM)
export(NUM.1)
export(NUM.1.NEG)
export(NUM.1.POS)
export(NUM.NEG)
export(NUM.POS)
export(abstract)
export(alike)
export(all_bw)
export(bench_mark)
export(nullify)
export(tev)
export(type_alike)
export(type_of)
export(vet)
export(vet_token)
export(vetr)
export(vetr_settings)
importFrom(methods,new)
importFrom(stats,median)
importFrom(utils,modifyList)
useDynLib(vetr, .registration=TRUE, .fixes="VALC_")
