% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\docType{class}
\name{vetr-interal}
\alias{vetr-interal}
\alias{vetr_bar-class}
\alias{vetr_baz-class}
\title{Test Objects}
\description{
Objects used for testing purposes only.
}
\keyword{internal}
